#ifndef _KGR_PATCH_BSC1133191_MM_H
#define _KGR_PATCH_BSC1133191_MM_H

#include <linux/mm.h>

/* New */
static inline int kgr_page_ref_count(struct page *page)
{
	return atomic_read(&page->_count);
}

/* New */
static inline __must_check bool kgr_try_get_page(struct page *page)
{
	struct page *head = compound_head(page);

	/*
	 * get_page() increases always head page's refcount, either directly or
	 * via __get_page_tail() for tail page, so we check that
	 */
	if (WARN_ON_ONCE(kgr_page_ref_count(head) <= 0))
		return false;

	get_page(page);
	return true;
}


/* from include/linux/mm.h */
extern int (*kgr_PageHeadHuge)(struct page *page_head);

/* resolve reference to non-EXPORTed PageHeadHuge() */
static inline bool kgr__compound_tail_refcounted(struct page *page)
{
	return PageAnon(page) && !PageSlab(page) && !kgr_PageHeadHuge(page);
}

/* resolve reference to non-EXPORTed PageHeadHuge() */
static inline bool kgr_compound_tail_refcounted(struct page *page)
{
	VM_BUG_ON_PAGE(!PageHead(page), page);
	return kgr__compound_tail_refcounted(page);
}

/* resolve reference to non-EXPORTed PageHeadHuge() */
static inline void kgr_get_huge_page_tail(struct page *page)
{
	/*
	 * __split_huge_page_refcount() cannot run from under us.
	 */
	VM_BUG_ON_PAGE(!PageTail(page), page);
	VM_BUG_ON_PAGE(page_mapcount(page) < 0, page);
	VM_BUG_ON_PAGE(atomic_read(&page->_count) != 0, page);
	if (kgr_compound_tail_refcounted(compound_head(page)))
		atomic_inc(&page->_mapcount);
}


#endif /* _KGR_PATCH_BSC1133191_MM_H */
